@@ -66,37 +66,48 @@ class Graph(core.GraphBase):
     cut_vertices = core.GraphBase.articulation_points
     blocks = core.GraphBase.biconnected_components
     evcent = core.GraphBase.eigenvector_centrality
+    vertex_disjoint_paths = core.GraphBase.vertex_connectivity
+    edge_disjoint_paths = core.GraphBase.edge_connectivity
+    cohesion = core.GraphBase.vertex_connectivity
+    adhesion = core.GraphBase.edge_connectivity
 
-    def __init__(self, n=1, edges=None, directed=None, \
-        graph_attrs=None, vertex_attrs=None, edge_attrs=None):
-        """Constructs a graph from scratch.
+    # Compatibility aliases
+    shortest_paths_dijkstra = core.GraphBase.shortest_paths
 
-        @param n: the number of vertices. Can be omitted.
-        @param edges: the edge list where every list item is a pair of integers.
+    def __init__(self, *args, **kwds):
+        """__init__(n=None, edges=None, directed=None, graph_attrs=None, vertex_attrs=None, edge_attrs=None)
+        
+        Constructs a graph from scratch.
+
+        @keyword n: the number of vertices. Can be omitted.
+        @keyword edges: the edge list where every list item is a pair of integers.
           If any of the integers is larger than M{n-1}, the number of vertices
           is adjusted accordingly.
-        @param directed: whether the graph should be directed
-        @param graph_attrs: the attributes of the graph as a dictionary.
-        @param vertex_attrs: the attributes of the vertices as a dictionary.
+        @keyword directed: whether the graph should be directed
+        @keyword graph_attrs: the attributes of the graph as a dictionary.
+        @keyword vertex_attrs: the attributes of the vertices as a dictionary.
           Every dictionary value must be an iterable with exactly M{n} items.
-        @param edge_attrs: the attributes of the edges as a dictionary. Every
+        @keyword edge_attrs: the attributes of the edges as a dictionary. Every
           dictionary value must be an iterable with exactly M{m} items where
           M{m} is the number of edges.
         """
-        # Check if n is a list. If so, that means that the number of vertices
-        # were omitted, so we should shift the whole parameter list with 1.
-        if isinstance(n, list) or isinstance(n, tuple):
-            edge_attrs = vertex_attrs
-            vertex_attrs = graph_attrs
-            graph_attrs = directed
-            directed = edges
-            edges = n
-            n = 1
-        edges = edges or []
-        directed = directed or False
-        graph_attrs = graph_attrs or {}
-        vertex_attrs = vertex_attrs or {}
-        edge_attrs = edge_attrs or {}
+        # Set up default values for the parameters. This should match the order in *args
+        kwd_order = ["n", "edges", "directed", "graph_attrs", "vertex_attrs", "edge_attrs"]
+        params = [1, [], False, {}, {}, {}]
+        # If the first argument is a list, assume that the number of vertices were omitted
+        args = list(args)
+        if len(args) > 0:
+            if isinstance(args[0], list) or isinstance(args[0], tuple):
+                args.insert(0, params[0])
+        # Override default parameters from args
+        params[:len(args)] = args
+        # Override default parameters from keywords
+        for idx, k in enumerate(kwd_order):
+            if k in kwds: params[idx] = kwds[k]
+        # Now, translate the params list to argument names
+        n, edges, directed, graph_attrs, vertex_attrs, edge_attrs = params
+
+        # Initialize the graph
         core.GraphBase.__init__(self, n, edges, directed)
         # Set the graph attributes
         for k, v in graph_attrs.iteritems():
@@ -227,7 +238,7 @@ class Graph(core.GraphBase):
         """
         return DyadCensus(GraphBase.dyad_census(self, *args, **kwds))
 
-    def eccentricity(self, nodes=None):
+    def eccentricity(self, vertices=None):
         """Calculates eccentricities for vertices with the given indices.
         
         Eccentricity is given as the reciprocal of the greatest distance
@@ -249,14 +260,14 @@ class Graph(core.GraphBase):
         distance_matrix = self.shortest_paths(mode=OUT)
         distance_maxs = map(max, distance_matrix)
         
-        if nodes is None:
+        if vertices is None:
             result = [1.0/x for x in distance_maxs]
         else:
-            result = [1.0/distance_maxs[idx] for idx in nodes]
+            result = [1.0/distance_maxs[idx] for idx in vertices]
 
         return result
 
-    def get_adjacency(self, type=GET_ADJACENCY_BOTH, attribute=None):
+    def get_adjacency(self, type=GET_ADJACENCY_BOTH, attribute=None, default=None):
         """Returns the adjacency matrix of a graph.
 
         @param type: either C{GET_ADJACENCY_LOWER} (uses the lower
@@ -265,11 +276,13 @@ class Graph(core.GraphBase):
           (uses both parts). Ignored for directed graphs.
         @param attribute: if C{None}, returns the ordinary adjacency
           matrix. When the name of a valid edge attribute is given
-          here, the matrix returned will contain C{None} at the places
-          where there is no edge or the value of the given attribute
-          where there is an edge. Multiple edges are not supported,
-          the value written in the matrix in this case will be
-          unpredictable.
+          here, the matrix returned will contain the default value 
+          at the places where there is no edge or the value of the
+          given attribute where there is an edge. Multiple edges are
+          not supported, the value written in the matrix in this case
+          will be unpredictable.
+        @param default: the default value written to the cells in the
+          case of adjacency matrices with attributes.
         @return: the adjacency matrix as a L{Matrix}.
         """
         if type != GET_ADJACENCY_LOWER and type != GET_ADJACENCY_UPPER and \
@@ -281,7 +294,7 @@ class Graph(core.GraphBase):
         if attribute is None: return Matrix(GraphBase.get_adjacency(self, type))
         if attribute not in self.es.attribute_names():
             raise ValueError, "Attribute does not exist"
-        data = [[None] * self.vcount() for _ in xrange(self.vcount())]
+        data = [[default] * self.vcount() for _ in xrange(self.vcount())]
 
         if not self.is_directed():
             if type == GET_ADJACENCY_BOTH:
@@ -333,6 +346,17 @@ class Graph(core.GraphBase):
         return [self.adjacent(idx, type) for idx in xrange(self.vcount())]
 
 
+    def mincut(self, capacity=None):
+        """mincut(capacity=None)
+
+        Returns a minimum cut in an undirected graph.
+
+        @param capacity: the edge capacities (weights). If C{None}, all
+          edges have equal weight.
+        @return: a L{Cut} object describing the minimum cut
+        """
+        return Cut(self, *GraphBase.mincut(self, capacity))
+
     def modularity(self, membership, weights=None):
         """modularity(membership, weights=None)
 
@@ -600,6 +624,8 @@ class Graph(core.GraphBase):
 
           - C{circle}, C{circular}: circular layout (see L{Graph.layout_circle})
 
+          - C{drl}: DrL layout for large graphs (see L{Graph.layout_drl})
+
           - C{fr}, C{fruchterman_reingold}: Fruchterman-Reingold layout
             (see L{Graph.layout_fruchterman_reingold}).
 
@@ -649,6 +675,63 @@ class Graph(core.GraphBase):
 
     # Auxiliary I/O functions
 
+    def write_adjacency(self, f, sep=" ", eol="\n", *args, **kwds):
+        """Writes the adjacency matrix of the graph to the given file
+
+        All the remaining arguments not mentioned here are passed intact
+        to L{Graph.get_adjacency}.
+
+        @param f: the name of the file to be written.
+        @param sep: the string that separates the matrix elements in a row
+        @param eol: the string that separates the rows of the matrix. Please
+          note that igraph is able to read back the written adjacency matrix
+          if and only if this is a single newline character
+        """
+        if not isinstance(f, file): f = file(f, "w")
+        matrix = self.get_adjacency(*args, **kwds)
+        for row in matrix:
+            f.write(sep.join(map(str, row)))
+            f.write(eol)
+        f.close()
+
+    def Read_Adjacency(klass, f, sep=None, comment_char = "#", attribute=None,
+        *args, **kwds):
+        """Constructs a graph based on an adjacency matrix from the given file
+
+        Additional positional and keyword arguments not mentioned here are
+        passed intact to L{Graph.Adjacency}.
+
+        @param f: the name of the file to be read or a file object
+        @param sep: the string that separates the matrix elements in a row.
+          C{None} means an arbitrary sequence of whitespace characters.
+        @param comment_char: lines starting with this string are treated
+          as comments.
+        @param attribute: an edge attribute name where the edge weights are
+          stored in the case of a weighted adjacency matrix. If C{None},
+          no weights are stored, values larger than 1 are considered as
+          edge multiplicities.
+        @return: the created graph"""
+        if not isinstance(f, file): f = file(f)
+        matrix, ri, weights = [], 0, {} 
+        for line in f:
+            line = line.strip()
+            if len(line) == 0: continue
+            if line.startswith(comment_char): continue
+            row = map(float, line.split(sep))
+            matrix.append(row)
+            ri += 1
+
+        f.close()
+
+        if attribute is None:
+            graph=klass.Adjacency(matrix, *args, **kwds)
+        else:
+            kwds["attr"] = attribute
+            graph=klass.Weighted_Adjacency(matrix, *args, **kwds)
+
+        return graph
+    Read_Adjacency=classmethod(Read_Adjacency)
+
     def write_graphmlz(self, f, compresslevel=9):
         """Writes the graph to a zipped GraphML file.
 
@@ -1239,7 +1322,26 @@ class Graph(core.GraphBase):
 
     def __plot__(self, context, bbox, palette, *args, **kwds):
         """Plots the graph to the given Cairo context in the given bounding box
-        
+       
+        The visual style of vertices and edges can be modified at three
+        places in the following order of precedence (lower indices override
+        higher indices):
+
+          1. Keyword arguments of this function (or of L{plot()} which is
+             passed intact to C{Graph.__plot__()}.
+
+          2. Vertex or edge attributes, specified later in the list of
+             keyword arguments.
+
+          3. igraph-wide plotting defaults (see
+             L{igraph.config.Configuration})
+
+          4. Built-in defaults.
+
+        E.g., if the C{vertex_size} keyword attribute is not present,
+        but there exists a vertex attribute named C{size}, the sizes of
+        the vertices will be specified by that attribute.
+
         Besides the usual self-explanatory plotting parameters (C{context},
         C{bbox}, C{palette}), it accepts the following keyword arguments:
         
@@ -1253,8 +1355,56 @@ class Graph(core.GraphBase):
           - C{margin}: the top, right, bottom, left margins as a 4-tuple.
             If it has less than 4 elements or is a single float, the elements
             will be re-used until the length is at least 4.
-            
-        TODO: vertex, edge parameters
+
+          - C{vertex_size}: size of the vertices. The corresponding vertex
+            attribute is called C{size}. The default is 10. Vertex sizes
+            are measured in the unit of the Cairo context on which igraph
+            is drawing.
+
+          - C{vertex_color}: color of the vertices. The corresponding vertex
+            attribute is C{color}, the default is red.  Colors can be
+            specified either by common X11 color names (see the source
+            code of L{igraph.colors} for a list of known colors), by
+            3-tuples of floats (ranging between 0 and 1 for the R, G and
+            B components), by CSS-style string specifications (C{#rrggbb})
+            or by integer color indices of the specified palette.
+
+          - C{vertex_shape}: shape of the vertices. Alternatively it can
+            be specified by the C{shape} vertex attribute. Possibilities
+            are: C{square}, {circle}, {triangle}, {triangle-down} or
+            C{hidden}. See the source code of L{igraph.drawing} for a
+            list of alternative shape names that are also accepted and
+            mapped to these.
+
+          - C{vertex_label}: labels drawn next to the vertices.
+            The corresponding vertex attribute is C{label}.
+
+          - C{vertex_label_dist}: distance of the midpoint of the vertex
+            label from the center of the corresponding vertex.
+            The corresponding vertex attribute is C{label_dist}.
+
+          - C{vertex_label_color}: color of the label. Corresponding
+            vertex attribute: C{label_color}. See C{vertex_color} for
+            color specification syntax.
+
+          - C{vertex_label_size}: font size of the label, specified
+            in the unit of the Cairo context on which we are drawing.
+            Corresponding vertex attribute: C{label_size}.
+
+          - C{vertex_label_angle}: the direction of the line connecting
+            the midpoint of the vertex with the midpoint of the label.
+            This can be used to position the labels relative to the
+            vertices themselves in conjunction with C{vertex_label_dist}.
+            Corresponding vertex attribute: C{label_angle}. The
+            default is C{-math.pi/4}.
+
+          - C{edge_color}: color of the edges. The corresponding edge
+            attribute is C{color}, the default is red. See C{vertex_color}
+            for color specification syntax.
+
+          - C{edge_width}: width of the edges in the default unit of the
+            Cairo context on which we are drawing. The corresponding
+            edge attribute is C{width}, the default is 1.
         """
         import colors
         import cairo
@@ -1287,7 +1437,10 @@ class Graph(core.GraphBase):
             layout = Layout(layout)
 
         sl, st, sr, sb = layout.bounding_box()
-        sw, sh = max(sr-sl, 1), max(sb-st, 1)
+        sw, sh = sr-sl, sb-st
+        if sw == 0 and sh == 0: sw, sh = 1, 1
+        if sw == 0: sw = sh
+        if sh == 0: sh = sw
         rx, ry = float(bbox.width-max_vertex_size-margin[1]-margin[3])/sw, \
           float(bbox.height-max_vertex_size-margin[0]-margin[2])/sh
         layout.scale(rx, ry)
@@ -1434,8 +1587,8 @@ class Graph(core.GraphBase):
           "gml":        ("Read_GML", "write_gml"),
           "dot":		(None, "write_dot"),
           "graphviz":	(None, "write_dot"),
-          "net":        ("Read_Pajek", None),
-          "pajek":      ("Read_Pajek", None),
+          "net":        ("Read_Pajek", "write_pajek"),
+          "pajek":      ("Read_Pajek", "write_pajek"),
           "dimacs":     ("Read_DIMACS", "write_dimacs"),
           "adjacency":  ("Read_Adjacency", "write_adjacency"),
           "adj":        ("Read_Adjacency", "write_adjacency"),
@@ -1449,6 +1602,7 @@ class Graph(core.GraphBase):
     _layout_mapping = {
         "circle": "layout_circle",
         "circular": "layout_circle",
+        "drl": "layout_drl",
         "fr": "layout_fruchterman_reingold",
         "fruchterman_reingold": "layout_fruchterman_reingold",
         "fr3d": "layout_fruchterman_reingold_3d",
@@ -1482,6 +1636,7 @@ class Graph(core.GraphBase):
     # After adjusting something here, don't forget to update the docstring
     # of Graph.layout if necessary!
 
+##############################################################
 
 class VertexSeq(core.VertexSeq):
     """Class representing a sequence of vertices in the graph.
@@ -1523,6 +1678,16 @@ class VertexSeq(core.VertexSeq):
       >>> g.vs.select(1,2)["weight"] = [10, 20]
       >>> g.vs["weight"]
       [0, 10, 20]
+
+    Some methods of the vertex sequences are simply proxy methods to the
+    corresponding methods in the L{Graph} object. One such example is
+    L{VertexSeq.degree()}:
+
+      >>> g=Graph.Tree(7, 2)
+      >>> g.vs.degree()
+      [2, 3, 3, 1, 1, 1, 1]
+      >>> g.vs.degree() == g.degree()
+      True
     """
 
     def select(self, *args, **kwds):
@@ -1666,6 +1831,7 @@ class VertexSeq(core.VertexSeq):
         """
         return self.select(*args, **kwds)
 
+##############################################################
 
 class EdgeSeq(core.EdgeSeq):
     """Class representing a sequence of edges in the graph.
@@ -1697,7 +1863,7 @@ class EdgeSeq(core.EdgeSeq):
       >>> [max(e.tuple) for e in g.es]
       [1, 2, 2]
       
-    The edge set can also be used as a dictionary where the keys are the
+    The edge sequence can also be used as a dictionary where the keys are the
     attribute names. The values corresponding to the keys are the values
     of the given attribute of every edge in the graph:
     
@@ -1710,6 +1876,16 @@ class EdgeSeq(core.EdgeSeq):
       >>> g.es["weight"] = range(3)
       >>> g.es["weight"]
       [0, 1, 2]
+
+    Some methods of the edge sequences are simply proxy methods to the
+    corresponding methods in the L{Graph} object. One such example is
+    L{EdgeSeq.is_multiple()}:
+
+      >>> g=Graph(3, [(0,1), (1,0), (1,2)])
+      >>> g.es.is_multiple()
+      [False, True, False]
+      >>> g.es.is_multiple() == g.is_multiple()
+      True
     """
 
     def select(self, *args, **kwds):
@@ -1854,6 +2030,65 @@ class EdgeSeq(core.EdgeSeq):
         """
         return self.select(*args, **kwds)
 
+##############################################################
+# Additional methods of VertexSeq and EdgeSeq that call Graph methods
+
+def _graphmethod(func=None, name=None):
+    """Auxiliary decorator
+    
+    This decorator allows some methods of L{VertexSeq} and L{EdgeSeq} to
+    call their respective counterparts in L{Graph} to avoid code duplication.
+
+    @param func: the function being decorated. This function will be
+      called on the results of the original L{Graph} method.
+      If C{None}, defaults to the identity function.
+    @param name: the name of the corresponding method in L{Graph}. If
+      C{None}, it defaults to the name of the decorated function.
+    @return: the decorated function
+    """
+    if name is None: name = func.__name__
+    method = getattr(Graph, name)
+
+    if callable(func):
+        def decorated(*args, **kwds):
+            self = args[0].graph
+            return func(args[0], method(self, *args, **kwds))
+    else:
+        def decorated(*args, **kwds):
+            self = args[0].graph
+            return method(self, *args, **kwds)
+
+    decorated.__name__ = name
+    decorated.__doc__ = """Proxy method to L{Graph.%(name)s()}
+
+This method calls the C{%(name)s()} method of the L{Graph} class
+restricted to this sequence, and returns the result.
+
+@see: Graph.%(name)s() for details.
+""" % { "name": name }
+
+    return decorated
+
+def _add_proxy_methods():
+    decorated_methods = {}
+    decorated_methods[VertexSeq] = \
+        ["degree", "betweenness", "bibcoupling", "closeness", "cocitation",
+        "constraint", "eccentricity", "get_shortest_paths", "maxdegree",
+        "pagerank", "shortest_paths", "similarity_dice", "similarity_jaccard",
+        "subgraph", "indegree", "outdegree", "isoclass"]
+    decorated_methods[EdgeSeq] = \
+        ["count_multiple", "is_loop", "is_multiple"]
+
+    for klass, methods in decorated_methods.iteritems():
+        for method in methods:
+            setattr(klass, method, _graphmethod(None, method))
+
+    setattr(EdgeSeq, "edge_betweenness", _graphmethod( \
+      lambda self, result: [result[i] for i in self.indices], "edge_betweenness"))
+
+_add_proxy_methods()
+
+##############################################################
 
 def read(filename, *args, **kwds):
     """Loads a graph from the given filename.
