@@ -99,7 +99,7 @@ class Clustering(object):
     def sizes(self, *args):
         """Returns the size of given clusters.
 
-        @param idxs: the cluster indices in which we are interested. If C{None},
+        @keyword idxs: the cluster indices in which we are interested. If C{None},
           defaults to all clusters
         """
         idxs = args
@@ -175,7 +175,7 @@ class OverlappingClustering(Clustering):
     def sizes(self, *args):
         """Returns the size of given clusters.
 
-        @param idxs: the cluster indices in which we are interested. If C{None},
+        @keyword idxs: the cluster indices in which we are interested. If C{None},
           defaults to all clusters
         """
         idxs = args
@@ -653,7 +653,7 @@ class VertexDendrogram(VertexClustering, Dendrogram):
             modularity = []
             n = graph.vcount()
             for step in xrange(min(n-1, len(merges))):
-                ms = community_to_membership(merges, graph.vcount(), maxidx)
+                ms = community_to_membership(merges, graph.vcount(), step)
                 modularity.append(graph.modularity(ms))
 
         if membership is None:
