@@ -42,7 +42,8 @@
  *
  * \param o a Python object to be converted
  * \param translation the translation table between strings and the
- *   enum values. Strings are treated as case-insensitive. The last
+ *   enum values. Strings are treated as case-insensitive, but it is
+ *   assumed that the translation table keys are lowercase. The last
  *   entry of the table must contain NULL values.
  * \param result the result is returned here. The default value must be
  *   passed in before calling this function, since this value is
@@ -53,24 +54,86 @@
 int igraphmodule_PyObject_to_enum(PyObject *o,
   igraphmodule_enum_translation_table_entry_t* table,
   int *result) {
-    char* s;
+    char *s, *s2;
+    int i, best, best_result, best_unique;
     
-    if (o==Py_None) return 0;
-    if (PyInt_Check(o)) return (int)PyInt_AsLong(o);
-    if (PyLong_Check(o)) return (int)PyLong_AsLong(o);
+    if (o == 0 || o==Py_None) return 0;
+    if (PyInt_Check(o)) { *result = (int)PyInt_AsLong(o); return 0; }
+    if (PyLong_Check(o)) { *result = (int)PyLong_AsLong(o); return 0; }
     if (!PyString_Check(o)) {
         PyErr_SetString(PyExc_TypeError, "int, long or string expected");
         return -1;
     }
     s=PyString_AsString(o);
+    /* Convert string to lowercase */
+    for (s2=s; *s2; s2++) *s2 = tolower(*s2);
+    best = 0; best_unique = 0; best_result = -1;
+    /* Search for matches */
     while (table->name != 0) {
-        if (strcasecmp(s, table->name) == 0) { *result = table->value; return 0; }
+        if (strcmp(s, table->name) == 0) { *result = table->value; return 0; }
+        for (i=0; s[i] == table->name[i]; i++);
+        if (i > best) {
+            best = i; best_unique = 1; best_result = table->value;
+        } else if (i == best) best_unique = 0;
         table++;
     }
+    if (best_unique) { *result = best_result; return 0; }
     PyErr_SetObject(PyExc_ValueError, o);
     return -1;
 }
 
+/**
+ * \ingroup python_interface_conversion
+ * \brief Converts a Python object to an igraph \c igraph_neimode_t
+ */
+int igraphmodule_PyObject_to_neimode_t(PyObject *o,
+  igraph_neimode_t *result) {
+  static igraphmodule_enum_translation_table_entry_t neimode_tt[] = {
+        {"in", IGRAPH_IN},
+        {"out", IGRAPH_OUT},
+        {"all", IGRAPH_ALL},
+        {0,0}
+    };
+
+  return igraphmodule_PyObject_to_enum(o, neimode_tt, (int*)result);
+}
+
+/**
+ * \ingroup python_interface_conversion
+ * \brief Converts a Python object to an igraph \c igraph_adjacency_t
+ */
+int igraphmodule_PyObject_to_adjacency_t(PyObject *o,
+  igraph_adjacency_t *result) {
+  static igraphmodule_enum_translation_table_entry_t adjacency_tt[] = {
+        {"directed", IGRAPH_ADJ_DIRECTED},
+        {"undirected", IGRAPH_ADJ_UNDIRECTED},
+        {"upper", IGRAPH_ADJ_UPPER},
+        {"lower", IGRAPH_ADJ_LOWER},
+        {"minimum", IGRAPH_ADJ_MIN},
+        {"maximum", IGRAPH_ADJ_MAX},
+        {"plus", IGRAPH_ADJ_PLUS},
+        {0,0}
+    };
+
+  return igraphmodule_PyObject_to_enum(o, adjacency_tt, (int*)result);
+}
+
+/**
+ * \ingroup python_interface_conversion
+ * \brief Converts a Python object to an igraph \c igraph_vconn_nei_t
+ */
+int igraphmodule_PyObject_to_vconn_nei_t(PyObject *o,
+  igraph_vconn_nei_t *result) {
+  static igraphmodule_enum_translation_table_entry_t vconn_nei_tt[] = {
+        {"error", IGRAPH_VCONN_NEI_ERROR},
+        {"infinity", IGRAPH_VCONN_NEI_INFINITY},
+        {"ignore", IGRAPH_VCONN_NEI_IGNORE},
+        {0,0}
+    };
+
+  return igraphmodule_PyObject_to_enum(o, vconn_nei_tt, (int*)result);
+}
+
 /**
  * \ingroup python_interface_conversion
  * \brief Converts a Python object to an igraph \c igraph_bliss_sh_t
@@ -90,6 +153,90 @@ int igraphmodule_PyObject_to_bliss_sh_t(PyObject *o,
   return igraphmodule_PyObject_to_enum(o, bliss_sh_tt, (int*)result);
 }
 
+/**
+ * \ingroup python_interface_conversion
+ * \brief Converts a Python object to an igraph \c igraph_degseq_t
+ */
+int igraphmodule_PyObject_to_degseq_t(PyObject *o,
+  igraph_degseq_t *result) {
+  static igraphmodule_enum_translation_table_entry_t degseq_tt[] = {
+        {"simple", IGRAPH_DEGSEQ_SIMPLE},
+        {"vl", IGRAPH_DEGSEQ_VL},
+        {"viger-latapy", IGRAPH_DEGSEQ_VL},
+        {0,0}
+    };
+
+  return igraphmodule_PyObject_to_enum(o, degseq_tt, (int*)result);
+}
+
+/**
+ * \brief Converts a Python object to an igraph \c igraph_integer_t
+ *
+ * Raises suitable Python exceptions when needed.
+ *
+ * \param object the Python object to be converted
+ * \param v the result is returned here
+ * \return 0 if everything was OK, 1 otherwise
+ */
+int igraphmodule_PyObject_to_integer_t(PyObject *object, igraph_integer_t *v) {
+  if (object == NULL) {
+  } else if (PyInt_Check(object)) {
+    long l = PyInt_AS_LONG((PyIntObject*)object);
+    *v=l;
+    return 0;
+  } else if (PyLong_Check(object)) {
+    double d = PyLong_AsDouble(object);
+    *v=d;
+    return 0;
+  } else if (PyNumber_Check(object)) {
+    PyObject *i = PyNumber_Int(object);
+    long l;
+    if (i == NULL) return 1;
+    l = PyInt_AS_LONG((PyIntObject*)i);
+    Py_DECREF(i);
+    *v = l;
+    return 0;
+  }
+  PyErr_BadArgument();
+  return 1;
+}
+
+/**
+ * \brief Converts a Python object to an igraph \c igraph_real_t
+ *
+ * Raises suitable Python exceptions when needed.
+ *
+ * \param object the Python object to be converted
+ * \param v the result is returned here
+ * \return 0 if everything was OK, 1 otherwise
+ */
+int igraphmodule_PyObject_to_real_t(PyObject *object, igraph_real_t *v) {
+  if (object == NULL) {
+  } else if (PyInt_Check(object)) {
+    long l = PyInt_AS_LONG((PyIntObject*)object);
+    *v=l;
+    return 0;
+  } else if (PyLong_Check(object)) {
+    double d = PyLong_AsDouble(object);
+    *v=d;
+    return 0;
+  } else if (PyFloat_Check(object)) {
+    double d = PyFloat_AS_DOUBLE((PyFloatObject*)object);
+    *v=d;
+    return 0;
+  } else if (PyNumber_Check(object)) {
+    PyObject *i = PyNumber_Int(object);
+    long l;
+    if (i == NULL) return 1;
+    l = PyInt_AS_LONG((PyIntObject*)i);
+    Py_DECREF(i);
+    *v = l;
+    return 0;
+  }
+  PyErr_BadArgument();
+  return 1;
+}
+
 /**
  * \ingroup python_interface_conversion
  * \brief Converts a Python object to an igraph \c igraph_vector_t
@@ -411,6 +558,71 @@ int igraphmodule_PyObject_float_to_vector_t(PyObject *list, igraph_vector_t *v)
   return 0;
 }
 
+/**
+ * \ingroup python_interface_conversion
+ * \brief Converts a Python list of objects to an igraph \c igraph_vector_bool_t
+ * The incoming \c igraph_vector_bool_t should be uninitialized. Raises suitable
+ * Python exceptions when needed.
+ * 
+ * \param list the Python list to be converted
+ * \param v the \c igraph_vector_bool_t containing the result
+ * \return 0 if everything was OK, 1 otherwise
+ */
+int igraphmodule_PyObject_to_vector_bool_t(PyObject *list,
+    igraph_vector_bool_t *v) {
+  PyObject *item;
+  int i, j;
+
+  if (PyString_Check(list) || PyUnicode_Check(list)) {
+    /* It is highly unlikely that a string (although it is a sequence) will
+     * provide us with integers or integer pairs */
+    PyErr_SetString(PyExc_TypeError, "expected a sequence or an iterable");
+    return 1;
+  }
+
+  if (!PySequence_Check(list)) {
+    /* try to use an iterator */
+    PyObject *it = PyObject_GetIter(list);
+    if (it) {
+      PyObject *item;
+      igraph_vector_bool_init(v, 0);
+      while ((item = PyIter_Next(it)) != 0) {
+        if (igraph_vector_bool_push_back(v, PyObject_IsTrue(item))) {
+          igraphmodule_handle_igraph_error();
+          igraph_vector_bool_destroy(v);
+          Py_DECREF(item);
+          Py_DECREF(it);
+          return 1;
+        }
+        Py_DECREF(item);
+      }
+      Py_DECREF(it);
+      return 0;
+    } else {
+      PyErr_SetString(PyExc_TypeError, "sequence or iterable expected");
+      return 1;
+    }
+    return 0;
+  }
+
+  j=PySequence_Size(list);
+  igraph_vector_bool_init(v, j);
+  for (i=0; i<j; i++) {
+    item=PySequence_GetItem(list, i);
+    if (item) {
+      VECTOR(*v)[i]=PyObject_IsTrue(item);
+	  Py_DECREF(item);
+    } else {
+      /* this should not happen, but we return anyway.
+       * an IndexError exception was set by PySequence_GetItem
+       * at this point */
+      igraph_vector_bool_destroy(v);
+      return 1;
+    }
+  }
+  return 0;
+}
+
 /**
  * \ingroup python_interface_conversion
  * \brief Converts an igraph \c igraph_vector_bool_t to a Python boolean list
@@ -1140,3 +1352,65 @@ int igraphmodule_PyObject_to_attribute_values(PyObject *o,
 
   return 0;
 }
+
+
+int igraphmodule_PyObject_to_drl_options_t(PyObject *obj,
+    igraph_layout_drl_options_t *options) {
+  if (obj == Py_None) {
+	igraph_layout_drl_options_init(options, IGRAPH_LAYOUT_DRL_DEFAULT);
+  } else if (PyString_Check(obj)) {
+    /* We have a string, so we are using a preset */
+	char* s=PyString_AsString(obj);
+	igraph_layout_drl_default_t def=IGRAPH_LAYOUT_DRL_DEFAULT;
+	if (strcmp(s, "default") == 0) def=IGRAPH_LAYOUT_DRL_DEFAULT;
+	else if (strcmp(s, "coarsen") == 0) def=IGRAPH_LAYOUT_DRL_COARSEN;
+	else if (strcmp(s, "coarsest") == 0) def=IGRAPH_LAYOUT_DRL_COARSEST;
+	else if (strcmp(s, "refine") == 0) def=IGRAPH_LAYOUT_DRL_REFINE;
+	else if (strcmp(s, "final") == 0) def=IGRAPH_LAYOUT_DRL_FINAL;
+	else {
+      PyErr_SetString(PyExc_ValueError, "unknown DrL template name. Must be one of: default, coarsen, coarsest, refine, final");
+      return 1;
+	}
+	if (igraph_layout_drl_options_init(options, def)) {
+	  igraphmodule_handle_igraph_error();
+	  return 1;
+	}
+  } else {
+    igraph_layout_drl_options_init(options, IGRAPH_LAYOUT_DRL_DEFAULT);
+#define CONVERT_DRL_OPTION(OPTION, TYPE) do { \
+      PyObject *o1; \
+      if (PyMapping_Check(obj)) { \
+        o1 = PyMapping_GetItemString(obj, #OPTION); \
+        igraphmodule_PyObject_to_##TYPE##_t(o1, &options->OPTION); \
+        Py_XDECREF(o1); \
+      } \
+      o1 = PyObject_GetAttrString(obj, #OPTION); \
+      igraphmodule_PyObject_to_##TYPE##_t(o1, &options->OPTION); \
+      Py_XDECREF(o1); \
+    } while (0)
+#define CONVERT_DRL_OPTION_BLOCK(NAME) do { \
+	  CONVERT_DRL_OPTION(NAME##_iterations, integer); \
+	  CONVERT_DRL_OPTION(NAME##_temperature, real); \
+	  CONVERT_DRL_OPTION(NAME##_attraction, real); \
+	  CONVERT_DRL_OPTION(NAME##_damping_mult, real); \
+    } while (0)
+	
+    CONVERT_DRL_OPTION(edge_cut, real);
+    CONVERT_DRL_OPTION_BLOCK(init);
+    CONVERT_DRL_OPTION_BLOCK(liquid);
+    CONVERT_DRL_OPTION_BLOCK(expansion);
+    CONVERT_DRL_OPTION_BLOCK(cooldown);
+    CONVERT_DRL_OPTION_BLOCK(crunch);
+    CONVERT_DRL_OPTION_BLOCK(simmer);
+
+#undef CONVERT_DRL_OPTION
+#undef CONVERT_DRL_OPTION_BLOCK
+
+    printf("cooldown_iterations = %.4f\n", options->cooldown_iterations);
+    printf("cooldown_damping_mult = %.4f\n", options->cooldown_damping_mult);
+    PyErr_Clear();
+	return 0;
+  }
+  return 0;
+}
+
