@@ -186,7 +186,7 @@ int igraphmodule_EdgeSeq_sq_length(igraphmodule_EdgeSeqObject* self) {
  * \brief Returns the item at the given index in the sequence
  */
 PyObject* igraphmodule_EdgeSeq_sq_item(igraphmodule_EdgeSeqObject* self,
-                       int i) {
+                       Py_ssize_t i) {
   igraph_t *g;
   long idx = -1;
   
@@ -313,9 +313,16 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values(igraphmodule_EdgeSeqObject*
 PyObject* igraphmodule_EdgeSeq_get_attribute_values_mapping(igraphmodule_EdgeSeqObject *self, PyObject *o) {
   /* Handle integer indices according to the sequence protocol */
   if (PyInt_Check(o)) return igraphmodule_EdgeSeq_sq_item(self, PyInt_AsLong(o));
-  if (PyTuple_Check(o) || PyList_Check(o)) {
-    /* Return a restricted EdgeSeq */
+  if (PyTuple_Check(o)) {
     return igraphmodule_EdgeSeq_select(self, o, NULL);
+  } else if (PyList_Check(o)) {
+    /* Return a restricted EdgeSeq */
+    PyObject *t = PyList_AsTuple(o);
+    PyObject *result;
+    if (!t) return NULL;
+    result = igraphmodule_EdgeSeq_select(self, t, NULL);
+    Py_DECREF(t);
+    return result;
   }
   return igraphmodule_EdgeSeq_get_attribute_values(self, o);
 }
@@ -352,9 +359,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
       for (i=0; i<n; i++) {
-        item = PyList_GetItem(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) return -1;
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, i, item)) {
           Py_DECREF(item);
           return -1;
@@ -366,10 +373,11 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
       list = PyList_New(n);
       if (list == 0) return -1;
       for (i=0; i<n; i++) {
-        item = PyList_GET_ITEM(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) { Py_DECREF(list); return -1; }
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         PyList_SET_ITEM(list, i, item);
+        /* PyList_SET_ITEM stole a reference to the item automatically */
       }
       if (PyDict_SetItem(dict, attrname, list)) {
         Py_DECREF(list);
@@ -389,7 +397,7 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
       return -1;
     }
     if (n != (long)igraph_vector_size(&es)) {
-      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of edges in the edge set");
+      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of edges in the edge sequence");
       igraph_vector_destroy(&es);
       return -1;
     }
@@ -398,9 +406,9 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
       for (i=0; i<n; i++) {
-        item = PyList_GetItem(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) { igraph_vector_destroy(&es); return -1; }
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, (long)VECTOR(es)[i], item)) {
           Py_DECREF(item);
           igraph_vector_destroy(&es);
@@ -419,13 +427,14 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
         PyList_SET_ITEM(list, i, Py_None);
       }
       for (i=0; i<n; i++) {
-        item = PyList_GET_ITEM(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) {
           igraph_vector_destroy(&es);
           Py_DECREF(list); return -1;
         }
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         PyList_SET_ITEM(list, (long)VECTOR(es)[i], item);
+        /* PyList_SET_ITEM stole a reference to the item automatically */
       }
       igraph_vector_destroy(&es);
       if (PyDict_SetItem(dict, attrname, list)) {
@@ -680,10 +689,10 @@ PyMethodDef igraphmodule_EdgeSeq_methods[] = {
  * edges by indices)
  */
 static PySequenceMethods igraphmodule_EdgeSeq_as_sequence = {
-  (inquiry)igraphmodule_EdgeSeq_sq_length,
+  (lenfunc)igraphmodule_EdgeSeq_sq_length,
   0,               /* sq_concat */
   0,               /* sq_repeat */
-  (intargfunc)igraphmodule_EdgeSeq_sq_item, /* sq_item */
+  (ssizeargfunc)igraphmodule_EdgeSeq_sq_item, /* sq_item */
   0,                                          /* sq_slice */
   0,                                          /* sq_ass_item */
   0,                                          /* sq_ass_slice */
@@ -699,7 +708,7 @@ static PySequenceMethods igraphmodule_EdgeSeq_as_sequence = {
  */
 static PyMappingMethods igraphmodule_EdgeSeq_as_mapping = {
     /* returns the number of edge attributes */
-    (inquiry) 0,
+    (lenfunc) 0,
     /* returns the values of an attribute by name */
     (binaryfunc) igraphmodule_EdgeSeq_get_attribute_values_mapping,
     /* sets the values of an attribute by name */
@@ -716,13 +725,41 @@ PyObject* igraphmodule_EdgeSeq_get_graph(igraphmodule_EdgeSeqObject* self,
   return (PyObject*)self->gref;
 }
 
+/**
+ * \ingroup python_interface_edgeseq
+ * Returns the indices of the edges in this edge sequence 
+ */
+PyObject* igraphmodule_EdgeSeq_get_indices(igraphmodule_EdgeSeqObject* self,
+  void* closure) {
+  igraphmodule_GraphObject *gr = self->gref;
+  igraph_vector_t es;
+  PyObject *result;
+
+  if (igraph_vector_init(&es, 0)) {
+    igraphmodule_handle_igraph_error();
+    return 0;
+  } 
+  if (igraph_es_as_vector(&gr->g, self->es, &es)) {
+    igraphmodule_handle_igraph_error();
+    igraph_vector_destroy(&es);
+    return 0;
+  }
+
+  result = igraphmodule_vector_t_to_PyList(&es, IGRAPHMODULE_TYPE_INT);
+  igraph_vector_destroy(&es);
+
+  return result;
+}
+
 /**
  * \ingroup python_interface_edgeseq
  * Getter/setter table for the \c igraph.EdgeSeq object
  */
 PyGetSetDef igraphmodule_EdgeSeq_getseters[] = {
   {"graph", (getter)igraphmodule_EdgeSeq_get_graph, NULL,
-      "The graph the edge sequence belongs to", NULL,
+      "The graph the edge sequence belongs to", NULL},
+  {"indices", (getter)igraphmodule_EdgeSeq_get_indices, NULL,
+      "The edge indices in this edge sequence", NULL,
   },
   {NULL}
 };
