@@ -182,7 +182,7 @@ int igraphmodule_VertexSeq_sq_length(igraphmodule_VertexSeqObject* self) {
  * \brief Returns the item at the given index in the sequence
  */
 PyObject* igraphmodule_VertexSeq_sq_item(igraphmodule_VertexSeqObject* self,
-                     int i) {
+                     Py_ssize_t i) {
   igraph_t *g;
   long idx = -1;
 
@@ -305,9 +305,17 @@ PyObject* igraphmodule_VertexSeq_get_attribute_values(igraphmodule_VertexSeqObje
 PyObject* igraphmodule_VertexSeq_get_attribute_values_mapping(igraphmodule_VertexSeqObject *self, PyObject *o) {
   /* Handle integer indices according to the sequence protocol */
   if (PyInt_Check(o)) return igraphmodule_VertexSeq_sq_item(self, PyInt_AsLong(o));
-  if (PyTuple_Check(o) || PyList_Check(o)) {
+  if (PyTuple_Check(o)) {
     /* Return a restricted VertexSeq */
     return igraphmodule_VertexSeq_select(self, o, NULL);
+  } else if (PyList_Check(o)) {
+    /* Return a restricted VertexSeq */
+    PyObject *t = PyList_AsTuple(o);
+    PyObject *result;
+    if (!t) return NULL;
+    result = igraphmodule_VertexSeq_select(self, t, NULL);
+    Py_DECREF(t);
+    return result;
   }
   return igraphmodule_VertexSeq_get_attribute_values(self, o);
 }
@@ -345,13 +353,13 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
       for (i=0; i<n; i++) {
-        item = PyList_GetItem(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) return -1;
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, i, item)) {
           Py_DECREF(item);
           return -1;
-        } /* PyList_SetItem stole a reference to the item automatically */ 
+        } /* PyList_SetItem stole a reference to the item automatically */  
       }
     } else if (values != 0) {
       /* We don't have attributes with the given name yet. Create an entry
@@ -359,10 +367,11 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
       list = PyList_New(n);
       if (list == 0) return -1;
       for (i=0; i<n; i++) {
-        item = PyList_GET_ITEM(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) { Py_DECREF(list); return -1; }
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         PyList_SET_ITEM(list, i, item);
+        /* PyList_SET_ITEM stole a reference to the item automatically */
       }
       if (PyDict_SetItem(dict, attrname, list)) {
         Py_DECREF(list);
@@ -382,7 +391,7 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
       return -1;
     }
     if (n != (long)igraph_vector_size(&vs)) {
-      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of vertices in the vertex set");
+      PyErr_SetString(PyExc_ValueError, "value list length must be equal to the number of vertices in the vertex sequence");
       igraph_vector_destroy(&vs);
       return -1;
     }
@@ -391,12 +400,12 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
       for (i=0; i<n; i++) {
-        item = PyList_GetItem(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) {
           igraph_vector_destroy(&vs);
           return -1;
         }
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, (long)VECTOR(vs)[i], item)) {
           Py_DECREF(item);
           igraph_vector_destroy(&vs);
@@ -418,13 +427,14 @@ int igraphmodule_VertexSeq_set_attribute_values_mapping(igraphmodule_VertexSeqOb
         PyList_SET_ITEM(list, i, Py_None);
       }
       for (i=0; i<n; i++) {
-        item = PyList_GET_ITEM(values, i);
+        item = PySequence_GetItem(values, i);
         if (item == 0) {
           igraph_vector_destroy(&vs);
           Py_DECREF(list); return -1;
         }
-        Py_INCREF(item);
+        /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         PyList_SET_ITEM(list, (long)VECTOR(vs)[i], item);
+        /* PyList_SET_ITEM stole a reference to the item automatically */
       }
       igraph_vector_destroy(&vs);
       if (PyDict_SetItem(dict, attrname, list)) {
@@ -661,6 +671,32 @@ PyObject* igraphmodule_VertexSeq_get_graph(igraphmodule_VertexSeqObject* self,
   return (PyObject*)self->gref;
 }
 
+/**
+ * \ingroup python_interface_vertexseq
+ * Returns the indices of the vertices in this vertex sequence 
+ */
+PyObject* igraphmodule_VertexSeq_get_indices(igraphmodule_VertexSeqObject* self,
+  void* closure) {
+  igraphmodule_GraphObject *gr = self->gref;
+  igraph_vector_t vs;
+  PyObject *result;
+
+  if (igraph_vector_init(&vs, 0)) {
+    igraphmodule_handle_igraph_error();
+    return 0;
+  } 
+  if (igraph_vs_as_vector(&gr->g, self->vs, &vs)) {
+    igraphmodule_handle_igraph_error();
+    igraph_vector_destroy(&vs);
+    return 0;
+  }
+
+  result = igraphmodule_vector_t_to_PyList(&vs, IGRAPHMODULE_TYPE_INT);
+  igraph_vector_destroy(&vs);
+
+  return result;
+}
+
 /**
  * \ingroup python_interface_vertexseq
  * Method table for the \c igraph.VertexSeq object
@@ -704,10 +740,10 @@ PyMethodDef igraphmodule_VertexSeq_methods[] = {
  * vertices by indices)
  */
 static PySequenceMethods igraphmodule_VertexSeq_as_sequence = {
-  (inquiry)igraphmodule_VertexSeq_sq_length,
+  (lenfunc)igraphmodule_VertexSeq_sq_length,
   0,               /* sq_concat */
   0,               /* sq_repeat */
-  (intargfunc)igraphmodule_VertexSeq_sq_item, /* sq_item */
+  (ssizeargfunc)igraphmodule_VertexSeq_sq_item, /* sq_item */
   0,                                          /* sq_slice */
   0,                                          /* sq_ass_item */
   0,                                          /* sq_ass_slice */
@@ -723,7 +759,7 @@ static PySequenceMethods igraphmodule_VertexSeq_as_sequence = {
  */
 static PyMappingMethods igraphmodule_VertexSeq_as_mapping = {
   /* this must be null, otherwise it f.cks up sq_length when inherited */
-  (inquiry) 0,
+  (lenfunc) 0,
   /* returns the values of an attribute by name */
   (binaryfunc) igraphmodule_VertexSeq_get_attribute_values_mapping,
   /* sets the values of an attribute by name */
@@ -738,6 +774,9 @@ PyGetSetDef igraphmodule_VertexSeq_getseters[] = {
   {"graph", (getter)igraphmodule_VertexSeq_get_graph, NULL,
       "The graph the vertex sequence belongs to", NULL,
   },
+  {"indices", (getter)igraphmodule_VertexSeq_get_indices, NULL,
+      "The vertex indices in this vertex sequence", NULL,
+  },
   {NULL}
 };
 
